import re


# Basic Items


def String(obj):
    return type(obj) == str


def Number(obj):
    return type(obj) == int


def List(obj):
    return type(obj) == list


def Dict(obj):
    return type(obj) == dict


# ID Items


def WeChatID(obj):
    return re.compile(r"^[a-zA-Z]{1}[-_a-zA-Z0-9]{5,19}$").match(obj)


def TencentQQID(obj):
    return re.compile(r"[1-9][0-9]{4,}").match(obj)


def PhoneID(obj):
    return re.compile(r"^1[34578]\d{9}$").match(obj)


def ChinaPhoneID(obj):
    return re.compile(r"\d{3}-\d{8}|\d{4}-\{7,8}").match(obj)


def CardID(obj):
    return re.compile(r"^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X|x)$").match(obj)


def ChinaPostID(obj):
    return re.compile(r"[1-9]\d{5}(?!\d)").match(obj)


# Address Items


def InternetProtocolAddress(obj):
    return re.compile(
        r"(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d).(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d).(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d).(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)"
    ).match(obj)


def URLAddress(obj):
    return re.compile(r"[a-zA-z]+://[^\s]*").match(obj)


def EmailAddress(obj):
    return re.compile(
        r"[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)"
    ).match(obj)


# Other Items


def ChineseOhhhYeah(obj):
    return re.compile("[\u4e00-\u9fa5]").match(obj)


__all__ = [
    "String",
    "Number",
    "List",
    "Dict",
    "WeChatID",
    "TencentQQID",
    "PhoneID",
    "ChinaPhoneID",
    "CardID",
    "ChinaPostID",
    "InternetProtocolAddress",
    "URLAddress",
    "EmailAddress",
    "ChineseOhhhYeah",
]



from flask import session

class Checker(object):
    def __init__(self, base={},session=None):
        self.base = base
        self.session = session

    def startCheck(self, obj):
        for nowColumnName in self.base:
            try:
                obj[nowColumnName]
            except:
                return [False, nowColumnName, "miss", obj]
        for nowColumn in obj:
            if (len(str(obj[nowColumn])) < self.base[nowColumn].smallest_str) or (
                len(str(obj[nowColumn])) > self.base[nowColumn].biggest_str
            ):
                return [False, nowColumn, "length", obj]
            if not self.base[nowColumn].checktype_func(obj[nowColumn]):
                return [False, nowColumn, "type", obj]
        session_dispatch = self.session(session=session,data=obj)
        if session_dispatch:
            return [True,"",obj]
        else:
            return [False,"","session",obj]


class Column(object):
    def __init__(self, checktype_func, biggest_str=25565, smallest_str=0):
        self.checktype_func = checktype_func
        self.biggest_str = biggest_str
        self.smallest_str = smallest_str


__all__ = ["Checker", "Column"]



from flask import current_app, _app_ctx_stack, request, sessions, Response
from functools import wraps

from .items import (
    String,
    Number,
    List,
    Dict,
    WeChatID,
    TencentQQID,
    PhoneID,
    ChinaPhoneID,
    CardID,
    ChinaPostID,
    InternetProtocolAddress,
    URLAddress,
    EmailAddress,
    ChineseOhhhYeah,
)

from .checker import Checker


class Was(object):
    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)

        self.String = String
        self.Number = Number
        self.List = List
        self.Dict = Dict
        self.WeChatID = WeChatID
        self.TencentQQID = TencentQQID
        self.PhoneId = PhoneID
        self.ChinaPhoneID = ChinaPhoneID
        self.CardID = CardID
        self.ChinaPostID = ChinaPostID
        self.InternetProtocolAddress = InternetProtocolAddress
        self.URLAddress = URLAddress
        self.EmailAddress = EmailAddress
        self.ChineseOhhhYeah = ChineseOhhhYeah

        self.checker = {}

    def init_app(self, app):
        app.config.setdefault("WASAPI_FORMAT", True)
        app.teardown_appcontext(self.teardown)

    def teardown(self, exception):
        ctx = _app_ctx_stack.top

    def handlerPost(self, checker):
        try:
            self.checker[checker]
        except:
            raise RuntimeError("Can find Checker.")
        data = request.form
        overdata = self.checker[checker].startCheck(data)
        return overdata

    def addChecker(self, namespace, obj):
        if not isinstance(obj, Checker):
            raise TypeError("Only Checker objects can be added.")
        self.checker[namespace] = obj

    def checkeout(self, hanlderChecker):
        def decorator(f):
            def handler(*args, **kwargs):
                print("a")
                handlerPost = self.handlerPost(hanlderChecker)
                return f(*((handlerPost,) + args), **kwargs)

            return handler

        return decorator

    def send(self, json={}, status=200):
        return Response(
            response=str(json), status=status, content_type="application/json"
        )


__all__ = ["Was"]



"""
Flask-Was
-------------

🍾 Flask extension for JSON API
"""
from setuptools import setup, find_packages


setup(
    name="flask_was",
    version="0.1.0",
    packages=find_packages(),
    install_requires=["Flask"],
)
